@@ -49,7 +49,6 @@
 #include <boost/algorithm/string.hpp>
 
 #include "cif++/CifUtils.hpp"
-#include "revision.hpp"
 
 namespace ba = boost::algorithm;
 namespace fs = std::filesystem;
@@ -65,9 +64,39 @@ extern int VERBOSE;
 
 std::string get_version_nr()
 {
-	std::ostringstream s;
-	write_version_string(s, false);
-	return s.str();
+	const std::regex
+		rxVersionNr1(R"(build-(\d+)-g[0-9a-f]{7}(-dirty)?)"),
+		rxVersionNr2(R"(libcifpp-version: (\d+\.\d+\.\d+))");
+
+#include "revision.hpp"
+
+	struct membuf : public std::streambuf
+	{
+		membuf(char *data, size_t length) { this->setg(data, data, data + length); }
+	} buffer(const_cast<char *>(kRevision), sizeof(kRevision));
+
+	std::istream is(&buffer);
+
+	std::string line, result;
+
+	while (getline(is, line))
+	{
+		std::smatch m;
+
+		if (std::regex_match(line, m, rxVersionNr1))
+		{
+			result = m[1];
+			if (m[2].matched)
+				result += '*';
+			break;
+		}
+
+		// always the first, replace with more specific if followed by the other info
+		if (std::regex_match(line, m, rxVersionNr2))
+			result = m[1];
+	}
+
+	return result;
 }
 
 // --------------------------------------------------------------------
@@ -94,11 +123,12 @@ const uint8_t kCharToLowerMap[256] =
 
 // --------------------------------------------------------------------
 
-bool iequals(const std::string &a, const std::string &b)
+bool iequals(std::string_view a, std::string_view b)
 {
 	bool result = a.length() == b.length();
-	for (auto ai = a.begin(), bi = b.begin(); result and ai != a.end() and bi != b.end(); ++ai, ++bi)
-		result = tolower(*ai) == tolower(*bi);
+	for (auto ai = a.begin(), bi = b.begin(); result and ai != a.end(); ++ai, ++bi)
+		result = kCharToLowerMap[uint8_t(*ai)] == kCharToLowerMap[uint8_t(*bi)];
+		// result = tolower(*ai) == tolower(*bi);
 	return result;
 }
 
@@ -111,7 +141,7 @@ bool iequals(const char *a, const char *b)
 	return result and *a == *b;
 }
 
-int icompare(const std::string &a, const std::string &b)
+int icompare(std::string_view a, std::string_view b)
 {
 	int d = 0;
 	auto ai = a.begin(), bi = b.begin();
@@ -164,7 +194,7 @@ std::string toLowerCopy(const std::string &s)
 
 // --------------------------------------------------------------------
 
-std::tuple<std::string, std::string> splitTagName(const std::string &tag)
+std::tuple<std::string, std::string> splitTagName(std::string_view tag)
 {
 	if (tag.empty())
 		throw std::runtime_error("empty tag");
